var def = require('../imports').default;
var THREE = def.THREE;
var Edge = def.Edge;
var Node = def.Node;
var OrbitControls = def.OrbitControls;
var d3 = def.d3;
var ee = def.ee;

/**
 * Calculates coordinates of mouse on canvas adjusted for padding
 * @param {*} event 
 * @param {*} currentElement 
 */
var relMouseCoords = function (event, currentElement) {
  var totalOffsetX = 0;
  var totalOffsetY = 0;
  var canvasX = 0;
  var canvasY = 0;

  do {
    totalOffsetX += currentElement.offsetLeft - currentElement.scrollLeft;
    totalOffsetY += currentElement.offsetTop - currentElement.scrollTop;
  }
  while(currentElement = currentElement.offsetParent)

  canvasX = event.pageX - totalOffsetX;
  canvasY = event.pageY - totalOffsetY;

  return {x:canvasX, y:canvasY}
}

/**
 * Add metadata to node
 * @param {*} node 
 */
var prepNode = function (node) {
  let nodeToAdd = node;
  if (!node.data) {
    node.data = {};
  }
  nodeToAdd.renderData = {
    label: node.data.label || "No Label",
    color: node.data.color || def.NODE_COLOR,
    hcolor: node.data.hcolor || def.NODE_HIGHLIGHT,
    shape: node.data.shape || def.NODE_SHAPE,
    linecolor: new THREE.Color(0.5, 0.5, 0.5),
    numYeast: Math.ceil(Math.random() * 8),
    size: node.data.size,
    tsize: node.data.tsize || def.TEXT_SIZE,
    x: 0,
    y: 0,
    vx: 0,
    vy: 0
  };
  return nodeToAdd;
}

/**
 * Creates a generic rectangle shape to use for selection and boundaries
 */
var make1x1Rect = function() {
  var rectShape = new THREE.Shape();
  rectShape.moveTo(0, 0);
  rectShape.lineTo(0, 1);
  rectShape.lineTo(1, 1);
  rectShape.lineTo(1, 0);
  rectShape.lineTo(0, 0);
  rectShape.lineTo(0, 1);
  return new THREE.ShapeGeometry(rectShape);
};


/**
 * Creates a centered 2x2 Rectangle
 */
var make2x2Rect = function() {
  var rectShape = new THREE.Shape();
  rectShape.moveTo(-1, -1);
  rectShape.lineTo(-1, 1);
  rectShape.lineTo(1, 1);
  rectShape.lineTo(1, -1);
  rectShape.lineTo(-1, -1);
  return new THREE.ShapeGeometry(rectShape);
};

/**
 *  find radius of circle given the volume
 * @param {*} volume
 */
var volToRadius = function (volume) {
  return Math.pow((volume / Math.PI) * (3.0 / 4.0), 1.0 / 3.0)
}
  

exports.relMouseCoords = relMouseCoords;
exports.prepNode = prepNode;
exports.make1x1Rect = make1x1Rect;
exports.make2x2Rect = make2x2Rect;
exports.volToRadius = volToRadius;